import setuptools

setuptools.setup(
    name='voussoirkit',
    packages=['voussoirkit'],
    version='0.0.25',
    author='voussoir',
    author_email='ethan@voussoir.net',
    description='voussoir\'s toolkit',
    url='https://github.com/voussoir/else',
    install_requires=['pyperclip']
)
